/*! Copyright (C) [2010] [David Mainzer]
 * 
 *  This program is free software; you can redistribute it and/or modify it under the terms 
 *  of the GNU General Public License as published by the Free Software Foundation; either 
 *  version 3 of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU General Public License for more details.
 *  
 *	You should have received a copy of the GNU General Public License along with this program; 
 *	if not, see <http://www.gnu.org/licenses/>.  
 */

import vrml.*;
import vrml.field.*;
import vrml.node.*;

import java.util.*;

public class FederMassSystem extends Script 
{
//	field SFNode objectPlane USE coord_Plane
	private SFNode m_planePoints;
//	field SFNode objectPlane2 USE coord_Geo
	private SFNode m_planeIndex;

	// list of mass-points (index-IDs)
	private Vector<Integer> m_massPointsIDs = new Vector<Integer>();
	
	private Vector<MassPoint> m_massPoints = new Vector<MassPoint>();
	
	long runtime   = (long) 0.0;										//!< simulation run time (in ms)
	long lastruntime = (long) 0.0;										//!< Zeit seit letztem frame
	long starttime = (long) 0.0;										//!< simulation start time (in ms)
	
	/*! \brief initialization of the hole mass-spring system
	 *  creates the springs
	 * 
	 */
	@Override
	public void initialize()
	{
		System.out.println("################################");
		System.out.println("## Spring-Mass-Sys using JAVA ##");
		System.out.println("################################");
		System.out.println("");
		System.out.println("Initializing java ... \n");
		
		// get the coordinates    	
		m_planePoints = (SFNode)getField("objectPlane");
    	MFVec3f cloth_Points = (MFVec3f)((Node)m_planePoints.getValue()).getExposedField("point");
    	
    	// get the index coordinates
    	m_planeIndex = (SFNode)getField("objectGeo");
    	MFInt32 cloth_Points_Index = (MFInt32)((Node)m_planeIndex.getValue()).getExposedField("coordIndex");
    	
    	// creating mass-points and connections between mass-points
    	for ( int i=0; i < cloth_Points_Index.getSize(); i=i+5 )
    	{
			int tmp_Index_0 = cloth_Points_Index.get1Value(i + 0);
			int tmp_Index_1 = cloth_Points_Index.get1Value(i + 1);
			int tmp_Index_2 = cloth_Points_Index.get1Value(i + 2);
			int tmp_Index_3 = cloth_Points_Index.get1Value(i + 3);

			if (! m_massPointsIDs.contains(tmp_Index_0))
			{
				m_massPointsIDs.add( new Integer(tmp_Index_0) );
				float tmp_point [] = new float[3];
	    		cloth_Points.get1Value(tmp_Index_0, tmp_point);
	    		
	    		MassPoint tmp_massPoint_0 = new MassPoint( tmp_Index_0, tmp_point );
	    		
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_1 );
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_3 );
	    		
	    		tmp_massPoint_0.addShearSpringConnection( tmp_Index_2 );
	    		
	    		m_massPoints.add( tmp_massPoint_0 );
			} else {
				Iterator<MassPoint> it = m_massPoints.iterator();
		    	
		    	while (it.hasNext()) {
		    		MassPoint tmp_massPoint = it.next(); 
		    		if ( tmp_massPoint.getID() == tmp_Index_0 )
		    		{
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_1 );
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_3 );
			    		
		    			tmp_massPoint.addShearSpringConnection( tmp_Index_2 );
		    		}
		    	}
			}
			
			if (! m_massPointsIDs.contains(tmp_Index_1))
			{
				m_massPointsIDs.add( new Integer(tmp_Index_1) );
				float tmp_point [] = new float[3];
	    		cloth_Points.get1Value(tmp_Index_1, tmp_point);
	    		
	    		MassPoint tmp_massPoint_0 = new MassPoint( tmp_Index_1, tmp_point );
	    		
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_2 );
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_0 );
	    		
	    		tmp_massPoint_0.addShearSpringConnection( tmp_Index_3 );
	    		
	    		m_massPoints.add( tmp_massPoint_0 );
			} else {
				Iterator<MassPoint> it = m_massPoints.iterator();
		    	
		    	while (it.hasNext()) {
		    		MassPoint tmp_massPoint = it.next(); 
		    		if ( tmp_massPoint.getID() == tmp_Index_1 )
		    		{
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_2 );
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_0 );
			    		
		    			tmp_massPoint.addShearSpringConnection( tmp_Index_3 );
		    		}
		    	}
			}
    	
			if (! m_massPointsIDs.contains(tmp_Index_2))
			{
				m_massPointsIDs.add( new Integer(tmp_Index_2) );
				float tmp_point [] = new float[3];
	    		cloth_Points.get1Value(tmp_Index_2, tmp_point);
	    		
	    		MassPoint tmp_massPoint_0 = new MassPoint( tmp_Index_2, tmp_point );
	    		
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_1 );
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_3 );
	    		
	    		tmp_massPoint_0.addShearSpringConnection( tmp_Index_0 );
	    		
	    		m_massPoints.add( tmp_massPoint_0 );
			} else {
				Iterator<MassPoint> it = m_massPoints.iterator();
		    	
		    	while (it.hasNext()) {
		    		MassPoint tmp_massPoint = it.next(); 
		    		if ( tmp_massPoint.getID() == tmp_Index_2 )
		    		{
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_1 );
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_3 );
			    		
		    			tmp_massPoint.addShearSpringConnection( tmp_Index_0 );
		    		}
		    	}
			}

			if (! m_massPointsIDs.contains(tmp_Index_3))
			{
				m_massPointsIDs.add( new Integer(tmp_Index_3) );
				float tmp_point [] = new float[3];
	    		cloth_Points.get1Value(tmp_Index_3, tmp_point);
	    		
	    		MassPoint tmp_massPoint_0 = new MassPoint( tmp_Index_3, tmp_point );
	    		
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_0 );
	    		tmp_massPoint_0.addStructuralSpringConnection( tmp_Index_2 );
	    		
	    		tmp_massPoint_0.addShearSpringConnection( tmp_Index_1 );
	    		
	    		m_massPoints.add( tmp_massPoint_0 );
			} else {
				Iterator<MassPoint> it = m_massPoints.iterator();
		    	
		    	while (it.hasNext()) {
		    		MassPoint tmp_massPoint = it.next(); 
		    		if ( tmp_massPoint.getID() == tmp_Index_3 )
		    		{
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_0 );
		    			tmp_massPoint.addStructuralSpringConnection( tmp_Index_2 );
			    		
		    			tmp_massPoint.addShearSpringConnection( tmp_Index_1 );
		    		}
		    	}
			}
    	}
    	
    	System.out.println("Number of mass points: " + m_massPoints.size() );
    	Iterator<MassPoint> it = m_massPoints.iterator();
    	while (it.hasNext()) {
    		it.next().printSpringConnections();
    	}
    	
		// set start time
		starttime = System.currentTimeMillis();
	}

	/** set uptime of the simulation (in ms)
	 * 
	 */
	public void setTimer ()
	{
		long timeInMillis = System.currentTimeMillis();
		runtime = timeInMillis - starttime;
	}

	@Override
    public void processEvent(Event e) 
	{
		setTimer();
		draw();
	}

	public void draw()
	{
		// get the coordinates    	
		MFVec3f cloth_Points = (MFVec3f)((Node)m_planePoints.getValue()).getExposedField("point");
    	
    	Iterator<MassPoint> it = m_massPoints.iterator();
    	while (it.hasNext()) {
    		MassPoint tmp_massPoint   = it.next(); 
    		int       tmp_massPointID = tmp_massPoint.getID();

			// bestimmen Sie die Krfte zwischen zwei Masse-Punkten!
    		
//    		228 -- 132 -- 231 -- 168 -- 232 -- 118 -- 243 -- 180 -- 244 -- 140
//  	     |      |      |      |      |      |      |      |      |      |
//    		130 --  21 -- 134 --  51 -- 133 --  18 -- 137 --  53 -- 138 --  22
//  		 |      |      |      |      |      |      |      |      |      |    		
//    		234 -- 136 -- 237 -- 174 -- 238 -- 120 -- 249 -- 186 -- 250 -- 144
//   		 |      |      |      |      |      |      |      |      |      |
//    		171 --  52 -- 173 --  68 -- 175 --  44 -- 185 --  71 -- 187 --  56
//    		 |      |      |      |      |      |      |      |      |      |
//    		236 -- 135 -- 239 -- 176 -- 240 -- 119 -- 251 -- 188 -- 252 -- 143
//    		 |      |      |      |      |      |      |      |      |      |
//    		114 --  17 -- 116 --  42 -- 115 --   8 -- 123 --  46 -- 124 --  19
//    		 |      |      |      |      |      |      |      |      |      |
//    		258 -- 147 -- 261 -- 198 -- 262 -- 127 -- 273 -- 210 -- 274 -- 155
//   		 |      |      |      |      |      |      |      |      |      |
//    		195 --  58 -- 197 --  74 -- 199 --  48 -- 209 --  77 -- 211 --  62
//  		 |      |      |      |      |      |      |      |      |      |    		
//    		260 -- 148 -- 263 -- 200 -- 264 -- 128 -- 275 -- 212 -- 276 -- 156
// 		 	 |      |      |      |      |      |      |      |      |      |    		
//    		146 --  23 -- 150 --  59 -- 149 --  20 -- 153 --  61 -- 154 --  24
// 		 	 |      |      |      |      |      |      |      |      |      |  
//    		266 -- 152 -- 269 -- 206 -- 270 -- 126 -- 281 -- 218 -- 282 -- 160
    		
			// ignore some Mass-points !!!
    		if ( tmp_massPointID != 228 && tmp_massPointID != 132 && tmp_massPointID != 231 && tmp_massPointID !=  168
    				 && tmp_massPointID !=  232 && tmp_massPointID !=  118 && tmp_massPointID !=  243 && tmp_massPointID != 180 
    				 && tmp_massPointID !=  244 && tmp_massPointID !=  140 && tmp_massPointID != 130 && tmp_massPointID !=   21 
    				 && tmp_massPointID !=  134 && tmp_massPointID !=   51 && tmp_massPointID !=  133 && tmp_massPointID !=   18 
    				 && tmp_massPointID !=  137 && tmp_massPointID !=   53 && tmp_massPointID !=  138 && tmp_massPointID !=   22
    	   			   )
    		{

			// hier soll die Schwerkraft auf alle Masse-Punkte, ausser die per ID ausgeschlossenen wirken!
    		
    		
    		}
    	}
    	
    	it = m_massPoints.iterator();
    	while (it.hasNext()) {
    		MassPoint tmp_massPoint   = it.next(); 
    		int       tmp_massPointID = tmp_massPoint.getID();
    		    		
    		tmp_massPoint.setOldPos();
    		tmp_massPoint.updateCurPos();
    		cloth_Points.set1Value(tmp_massPointID, tmp_massPoint.getX(), tmp_massPoint.getY(), tmp_massPoint.getZ());
    	}    	
    	
	}


}
